import com.haxepunk.HXP;
import com.haxepunk.Entity;
import com.haxepunk.graphics.*;
import com.haxepunk.masks.*;

class Door extends Entity {
	var part1:Stamp;
	var part2:Stamp;
	var mask1:Hitbox;
	var mask2:Hitbox;
	var horizontal:Bool;
	var canvas:Canvas;
	var state:Int = 0;
	var moving:Int = 0;

	public function new(x:Float, y:Float, w:Float, h:Float, horizontal:Bool, open:Bool) {
		super(x, y);
		var iw:Int = Std.int(w);
		var ih:Int = Std.int(h);
		state = open? iw>>1: 0;
		this.horizontal = horizontal;
		if(horizontal) {
			part1 = new Stamp("graphics/door_h1.png");
			part2 = new Stamp("graphics/door_h2.png");
			mask1 = new Hitbox(iw, ih);
			mask2 = new Hitbox(iw, ih);
			mask = new Masklist( [ mask1, mask2] );
			state = open? iw>>1: 0;
		} else {
			part1 = new Stamp("graphics/door_v1.png");
			if(iw < part1.width) {
				x -= (part1.width - iw) / 2;
				iw = part1.width;
			}
			mask = mask1 = new Hitbox(iw, ih);
			state = open? ih: 0;
		}
		graphic = canvas = new Tilemap("graphics/door_h1.png", iw, ih, 4,4);	// use tilemap as canvas is broken
		change(state);
		type = "solid";
	}

	public function change(o:Int) {
		HXP.rect.x = HXP.rect.y = 0;
		HXP.rect.width = canvas.width;
		HXP.rect.height = canvas.height;
		canvas.fill(HXP.rect, 0, 0);

		if(horizontal) {
			var half:Int = canvas.width >> 1;
			var offset:Int = half - (part1.width>>1);
			mask1.width = mask2.width = half - o;
			mask2.x = half + o;

			canvas.drawGraphic(offset - o, 0, part1);
			canvas.drawGraphic(offset + o, 0, part2);
		} else {
			var offset:Int = canvas.height - part1.height;
			mask1.height = canvas.height - o;
			canvas.drawGraphic(0, offset - o, part1);
		}
	}

	public function trigger() {
		if(moving==0) moving = state==0? 1: -1;
		else moving = -moving;
	}

	public override function update() {
		if(moving!=0) {
			var max:Int = horizontal? canvas.width>>1: canvas.height;
			state += moving * 2;
			if(state<=0) state = moving = 0;
			if(state>=max) { state=max; moving=0; }
			change(state);
		}
	}

}

