import com.haxepunk.Scene;
import com.haxepunk.HXP;
import com.haxepunk.Sfx;
import com.haxepunk.Entity;
import com.haxepunk.graphics.*;
import com.haxepunk.utils.*;
import openfl.Assets;

class Level extends Scene {
	public var index:Int;
	public var player:Player;
	public static var music:Sfx;
	public var winSound: Sfx;

	public var state:Int=0;
	public var fade:Entity;
	
	public function new(index:Int) {
		super();
		// Load level
		this.index = index;
		var source:String = "level"+index+".tmx";

		if(!Assets.exists(source, AssetType.TEXT)) return;

		


		// Tile map
		var map:Tiled = new Tiled(source);
		var g:Tilemap = map.createTilemap("Tiles", "Tiles");
		addGraphic(g);

		var stuff:Array<Entity> = [];

		// Turn crumbly tiles into entities
		for(x in 0...g.columns) {
			for(y in 0...g.rows) {
				var t:Int = g.getTile(x, y);
				if(t==1 || t==2) {
					g.setTile(x, y, 3);
					stuff.push( new Broken(x*24, y*24, 3-t) );
				}
			}
		}



		addMask( map.createClipmap("Tiles", "Tiles", 0), "solid" );

		// Objects
		for(e in map.getObjects(null, "Objects")) {
			if(e.type == "Grow") add( new Potion(e.x, e.y, 1) );
			if(e.type == "Shrink") add( new Potion(e.x, e.y, -1) );
			if(e.type == "Exit") {
				var e:Entity = new Entity(e.x, e.y+e.height-33, new Stamp("graphics/door.png"));
				e.type = "exit";
				e.setHitbox(4,12,-10,-10);
				add(e);
			}
			if(e.type == "Switch") {
				stuff.push(new Button(e.x, e.y + e.height, 0) );
			}
			if(e.type == "Door") stuff.push( new Door(e.x, e.y, e.width, e.height, false, false) );
			if(e.type == "DoorH") stuff.push( new Door(e.x, e.y, e.width, e.height, true, false) );
		}

		// Links
		for(e in map.getObjects("", "Links")) {
			if(e.points!=null) {
				var s:Entity = find(e.points[0].x + e.x, e.points[0].y+e.y, stuff);
				var t:Entity = find(e.points[1].x + e.x, e.points[1].y+e.y, stuff);
				if(s!=null && t!=null && Std.is(s, Button)) {
					cast(s, Button).link(t);
				} else trace("Link failed", s, t);
			}
		}

		addList(stuff);

		// Player
		player = new Player(100, 100);
		for(p in map.getObjects("Player")) {
			player.x = p.x;
			player.y = p.y+p.height;
		}
		add(player);

		// Fadey thing
		fade = new Entity(0, 0, Image.createRect(HXP.width, HXP.height, 0));
		add(fade);



		winSound = new Sfx("audio/win.mp3");
		if(music == null) {
			music = new Sfx("audio/music.mp3");
			music.loop(0.1);
		}
	}
	private function find(x:Float, y:Float, list:Array<Entity>): Entity {
		var d:Float = 16;
		var r:Entity = null;
		for(e in list) {
			var cd:Float = e.distanceToPoint(x, y, true);
			if(cd < d) {
				d = cd;
				r = e;
			}
		}
		return r;
	}
	

	public override function begin() {
	}

	public override function update() {
		super.update();

		// Restart
		if(Input.pressed(Key.R)) {
			state = -1;
			fade.visible = true;
		}

		// Exit
		if(player != null && player.collide("exit", player.x, player.y)!=null) {
			winSound.play();
			fade.visible = true;
			state = 1;
		}

		// Mute music
		if(Input.pressed(Key.M)) {
			if(music.playing) music.stop();
			else music.loop(0.1);
		}

		// Debug Level Skip
		if(Input.pressed(Key.F6) && index>1) HXP.scene = new Level(index-1);
		if(Input.pressed(Key.F7)) HXP.scene = new Level(index+1);

		// Fade
		if(fade!=null && fade.visible) {
			var img:Image = cast fade.graphic;
			img.alpha += 0.02 * (state==0? -1: 1);
			if(img.alpha<=0) fade.visible = false;
			if(img.alpha>=1) {
				if(state < 0) HXP.scene = new Level(index);
				if(state > 0) HXP.scene = new Level(index+1);
			}
		}
	}

}

