import com.haxepunk.Entity;
import com.haxepunk.Sfx;
import com.haxepunk.graphics.*;
import com.haxepunk.utils.*;
import com.haxepunk.masks.*;



class Player extends Entity {
	public var speed : Float = 2;
	public var fall: Float = 0;
	public var jump: Float = 0;
	public var jumpHeight: Float = 4;
	public var climbSpeed: Float =  2;
	public var climb: Int = 0;
	public var size: Int = 1;
	public var scale: Float = 1;

	public var drinkSound: Sfx;
	public var growSound: Sfx;
	public var shrinkSound: Sfx;
	public var jumpSound: Sfx;

	public function new(x:Float, y:Float) {
		super(x, y);
		//graphic = Image.createRect(4, 18, 0xffffff);
		graphic = new Image("graphics/player.png");
		setHitbox(4, 18, 2, 18);
	//	mask = new Hitbox(4, 18, 2, 18);
		type = "player";
		changeScale(1);

		Input.define("lt", [Key.LEFT]);
		Input.define("rt", [Key.RIGHT]);
		drinkSound = new Sfx("audio/drink.mp3");
		growSound = new Sfx("audio/grow.mp3");
		shrinkSound = new Sfx("audio/shrink.mp3");
		jumpSound = new Sfx("audio/jump.mp3");
	}

	public function changeScale(s: Float) {
		var img:Image = cast graphic;
		scale = s;
		graphic.x = -5 * s;
		graphic.y = -18 * s;
		setHitbox(Std.int(4*s), Std.int(18*s), Std.int(2*s), Std.int(18*s));
		img.scale  = s / 3;
		speed      = 0.4 + 1.8 * s;
		jumpHeight = 4 * s;
		climbSpeed = 1 + 1 * s;
		if(collide("solid", x, y)!=null) {
			for(i in 1...4) {
				if(collide("solid", x+i, y)==null) { x+=i; break; }
				if(collide("solid", x-i, y)==null) { x-=i; break; }
			}
		}
	}

	public override function update() {
		// Upate scale
		if(scale < size) changeScale(Math.min(scale * 1.1, size));
		else if(scale > size) changeScale(Math.max(scale * 0.9, size));

		if(cast(scene, Level).state != 0) return;

		// Movement
		var move: Int = 0;
		if(Input.check("lt")) move = -1;
		if(Input.check("rt")) move = 1;

		if(move!=0) cast(graphic, Image).flipped = move<0;

		if(fall<0 && move==0) fall = 0;
		if(move==0) climb = 0;
		if(climb==1) {
			var lx:Float = x;
			moveBy(move, -climbSpeed, "solid");
			if(x!=lx) climb = 2;
			return;
		}
		
		moveBy(move*speed, fall, "solid");

		var onGround = fall==0;

		if(collide("solid", x, y+1)==null) fall += 0.8 * size;
		else {
			fall = 0;
			jump = 0;
			if(climb==2) climb = 0;
			y = Math.floor(y);
		}

		// Jump
		if(onGround && scene.collidePoint("solid", x+move*(width/2-1), y+1)==null) {
			if(move!=0 && jump==0 && climb==0) {
				jump = 10;
				fall = -jumpHeight;
				jumpSound.play();
			}
		}

		// Climb detection
		if(move!=0 /*&& fall==0*/ && collide("solid", x+move, y)!=null) {
			// can we climb ?
			for(i in 1...size+1) {
				var extra:Int = climb>0 || (jump==0 && onGround)? 0: 6*size; // Extra height when on ground
				if(collide("solid", x+move, y-24*i+extra)==null) climb = 1;
			}
		}

		// Potions
		var potion: Entity = collide("potion", x, y);
		if(potion != null) {
			var e:Int = cast(potion, Potion).effect;
			if(size>1 && e<0) {
				--size;
				shrinkSound.play();
			}
			else if(size<3 && e>0) {
				++size;
				growSound.play();
			}
			else drinkSound.play();
			scene.remove(potion);
		}
	}
}

